import { Component, inject, OnInit } from '@angular/core';
import { ApiAuthenticationService } from '../../shared/services/api/api-authentication.service';
import { MatCardModule } from '@angular/material/card';
import { MatSelectModule } from '@angular/material/select';
import { HttpClient } from '@angular/common/http';
import { environment } from '../../../environments/environment';
import EventService from '../../shared/services/EventService';
import { HermesClientService } from '../../hermes-client.service';
import { FormControl, ReactiveFormsModule } from '@angular/forms';
import { UserService } from '../../shared/services/user.service';
import { AsyncPipe } from '@angular/common';
import { ExcludeByIdPipe } from '../../shared/pipes/exclude-by-id.pipe';

@Component({
  selector: 'impersonation',
  standalone: true,
  imports: [
    AsyncPipe,
    ExcludeByIdPipe,
    MatCardModule,
    MatSelectModule,
    ReactiveFormsModule,
  ],
  templateUrl: './impersonation.component.html',
  styleUrl: './impersonation.component.scss'
})
export class ImpersonationComponent implements OnInit {
  private readonly client = inject(HermesClientService);
  private readonly userService = inject(UserService);
  private readonly events = inject(EventService);
  private readonly http = inject(HttpClient);

  readonly auth = inject(ApiAuthenticationService);

  impersonationControl = new FormControl<string>(this.auth.getUserId());
  users$ = this.userService.fetch();

  ngOnInit(): void {
    if (!this.auth.isAdmin()) {
      return;
    }

    this.users$.subscribe(users => {
      const id = this.auth.getImpersonatedId();
      if (id && users.find(u => u.id == id)) {
        this.impersonationControl.setValue(id);
      }
    });

    this.impersonationControl.valueChanges.subscribe((impersonationId) => {
      if (impersonationId == this.auth.getImpersonatedId())
        return;

      if (impersonationId == this.auth.getUserId()) {
        this.http.delete(environment.API_HOST + '/admin/impersonate', {
          headers: {
            'Authorization': 'Bearer ' + localStorage.getItem('jwt')
          },
          body: {
            impersonation: impersonationId
          }
        }).subscribe(async (data: any) => {
          this.client.disconnect(true);
          this.events.emit('impersonation', impersonationId);
        });
      } else {
        this.http.put(environment.API_HOST + '/admin/impersonate', {
          impersonation: impersonationId
        }, {
          headers: {
            'Authorization': 'Bearer ' + localStorage.getItem('jwt')
          }
        }).subscribe(async (data: any) => {
          this.client.disconnect(true);
          this.events.emit('impersonation', impersonationId);
        });
      }
    });
  }

  public isAdmin() {
    return this.auth.isAdmin();
  }

  public getUsername() {
    return this.auth.getUsername();
  }
}