import { Component, input, Input, model, OnInit } from '@angular/core';
import { FormControl, ReactiveFormsModule } from '@angular/forms';
import { MatAutocompleteModule } from '@angular/material/autocomplete';
import { MatFormFieldModule } from '@angular/material/form-field';
import TwitchRedemption from '../../shared/models/twitch-redemption';
import { MatInputModule } from '@angular/material/input';
import { toTwitchRedemptionArray } from '../../shared/transformers/twitch-redemption.transformer';

@Component({
  selector: 'twitch-redemption-dropdown',
  imports: [MatAutocompleteModule, MatFormFieldModule, MatInputModule, ReactiveFormsModule],
  templateUrl: './twitch-redemption-dropdown.component.html',
  styleUrl: './twitch-redemption-dropdown.component.scss'
})
export class TwitchRedemptionDropdownComponent implements OnInit {
  @Input() formControl = new FormControl<TwitchRedemption | string | undefined>(undefined);
  @Input() errorMessages: { [errorKey: string]: string } = {};

  twitchRedemptions = input.required({
    transform: toTwitchRedemptionArray
  });
  twitchRedemptionId = model<string | undefined>();
  search = input<boolean>(false);

  errorMessageKeys: string[] = [];


  ngOnInit(): void {
    if (this.twitchRedemptions() && this.twitchRedemptionId()) {
      const redemption = this.twitchRedemptions()!.find(r => r.id == this.twitchRedemptionId());
      this.formControl.setValue(redemption);
    }
    this.errorMessageKeys = Object.keys(this.errorMessages);
  }

  get filteredRedemptions() {
    const value = this.formControl.value;
    if (this.twitchRedemptions() && typeof (value) == 'string') {
      return this.twitchRedemptions()!.filter(r => r.title.toLowerCase().includes(value.toLowerCase()));
    }
    return this.twitchRedemptions();
  }

  select(event: TwitchRedemption) {
    this.twitchRedemptionId.set(event.id);
  }

  input() {
    if (this.search() && typeof this.formControl.value == 'string') {
      this.twitchRedemptionId.set(this.formControl.value);
    }
  }

  blur() {
    if (this.filteredRedemptions == null)
      return;

    if (!this.search && typeof this.formControl.value == 'string') {
      const name = this.formControl.value;
      const nameLower = name.toLowerCase();
      let newValue: TwitchRedemption | undefined;
      const filtered = this.filteredRedemptions?.filter(a => a.title.toLowerCase() == nameLower);
      if (filtered.length > 1) {
        newValue = filtered.find(a => a.title == name);
      } else if (filtered.length == 1) {
        newValue = filtered[0];
      }
      if (newValue && newValue.id != this.formControl.value) {
        this.formControl.setValue(newValue);
        this.twitchRedemptionId.set(newValue.id);
      } else if (!newValue)
        this.twitchRedemptionId.set(undefined);
    }
  }

  displayFn(value: any) {
    return value?.title;
  }
}
