import { Component, inject, OnDestroy } from '@angular/core';
import { FilterListComponent } from "../filter-list/filter-list.component";
import { MatButtonModule } from '@angular/material/button';
import { MatIconModule } from '@angular/material/icon';
import { Filter, FilterFlag } from '../../shared/models/filter';
import { ActivatedRoute } from '@angular/router';
import { FilterItemEditComponent } from '../filter-item-edit/filter-item-edit.component';
import { MatDialog } from '@angular/material/dialog';
import TtsFilterService from '../../shared/services/tts-filter.service';
import { Subscription } from 'rxjs';

@Component({
  selector: 'filters',
  standalone: true,
  imports: [
    FilterListComponent,
    MatButtonModule,
    MatIconModule
  ],
  templateUrl: './filters.component.html',
  styleUrl: './filters.component.scss'
})
export class FiltersComponent implements OnDestroy {
  private readonly filterService = inject(TtsFilterService);
  private readonly route = inject(ActivatedRoute);
  private readonly subscriptions: (Subscription | undefined)[] = [];

  readonly dialog = inject(MatDialog);

  items: Filter[] = [];


  constructor() {
    this.route.data.subscribe(data => this.items = data['filters'] || []);

    this.subscriptions.push(this.filterService.delete$?.subscribe(d => this.items = this.items.filter(a => a.id != d.request.data.id)));
  }

  ngOnDestroy() {
    this.subscriptions.filter(s => !!s).forEach(s => s.unsubscribe());
  }

  openDialog(): void {
    this.dialog.open(FilterItemEditComponent, {
      data: { id: '', user_id: '', search: '', replace: '', flag: FilterFlag.None },
    });
  }
}